@@ -23,6 +23,30 @@ context "Rabl::Engine" do
     asserts_topic.assigns :_view_path
   end
 
+  context "#request_format" do
+    context "is json by default" do
+      setup do
+        template = RablTemplate.new("code") { "" }
+        template.render(Object.new)
+        engine = template.instance_eval('@engine')
+        engine.instance_eval('@_options')[:format]
+      end
+
+      asserts_topic.equals('json')
+    end
+
+    context "with a specified format" do
+      setup do
+        template = RablTemplate.new("code", format: 'xml') { "" }
+        template.render(Object.new)
+        engine = template.instance_eval('@engine')
+        engine.instance_eval('@_options')[:format]
+      end
+
+      asserts_topic.equals('xml')
+    end
+  end
+
   context "#cache" do
     context "with cache" do
       setup do
@@ -302,7 +326,7 @@ context "Rabl::Engine" do
         template.render(scope)
       end.equals "{\"user\":{\"name\":\"leo\",\"users\":[{\"user\":{\"city\":\"UNO\"}},{\"user\":{\"city\":\"DOS\"}}]}}"
 
-      asserts "child chooses name based on symbol if no elements" do
+      asserts "that it chooses a name based on symbol if no elements" do
         template = rabl %{
           object @bar => :bar
           child(:foos) { attribute :city }
@@ -314,6 +338,18 @@ context "Rabl::Engine" do
         template.render(scope)
       end.equals "{\"bar\":{\"foos\":[]}}"
 
+      asserts "that it chooses a name based on symbol if nil" do
+        template = rabl %{
+          object @bar => :bar
+          child(:foos) { attribute :city }
+        }
+        scope = Object.new
+        bar = Object.new
+        stub(bar).foos { nil }
+        scope.instance_variable_set :@bar, bar
+        template.render(scope)
+      end.equals "{\"bar\":{\"foos\":null}}"
+
       asserts "it allows suppression of root node for child collection" do
         template = rabl %{
           object @user
@@ -708,6 +744,7 @@ context "Rabl::Engine" do
         File.open(tmp_path + "test.json.rabl", "w") do |f|
           f.puts %q{
             attributes :age
+            node(:city) { "Gotham" } if locals[:show_city]
           }
         end
       end
@@ -723,6 +760,17 @@ context "Rabl::Engine" do
         JSON.parse(template.render(scope))
       end.equals JSON.parse("{\"name\":\"leo\",\"age\":12}")
 
+      asserts "that it can be passed locals" do
+        template = rabl %{
+          object @user
+          attribute :name
+          extends 'test', :locals => { :show_city => true }
+        }
+        scope = Object.new
+        scope.instance_variable_set :@user, User.new(:name => 'leo', :age => 12)
+        JSON.parse(template.render(scope))
+      end.equals JSON.parse("{\"name\":\"leo\",\"age\":12,\"city\":\"Gotham\"}")
+
       asserts "that it can be passed conditionals" do
         template = rabl %{
           object @user
